package com.dragon.tj.portal.auth.service;

import com.dragon.tj.portal.auth.model.LoginUser;
import com.dragon.tj.portal.auth.web.entity.SysUser;
import com.dragon.tj.portal.auth.web.service.SysUserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.Objects;
import java.util.Set;

/**
 * 用户验证处理
 *
 * @author tienchin
 */
@Service
public class MyUserDetailsService implements UserDetailsService {
    private static final Logger LOGGER = LoggerFactory.getLogger(MyUserDetailsService.class);

    @Autowired
    private SysUserService userService;
    @Autowired
    private SysPermissionService sysPermissionService;


    @Override
    public UserDetails loadUserByUsername(String idCard) throws UsernameNotFoundException {
        SysUser user = userService.getUserById(idCard);
        if (Objects.isNull(user)) {
            throw new UsernameNotFoundException(idCard);
        }
        return createLoginUser(user);
    }

    public UserDetails createLoginUser(SysUser sysUser) {
        Set<String> permissions = sysPermissionService.getMenuPermission(sysUser);
        return new LoginUser(sysUser.getIdcard(), sysUser.getOrgCode(), sysUser, new HashSet<>(permissions));
    }
}
