package com.dragon.tj.portal.controller;

import com.dragon.tj.portal.auth.model.LoginUser;
import com.dragon.tj.portal.auth.util.SecurityUtils;
import com.dragon.tj.portal.common.base.R;
import com.dragon.tj.portal.entity.BusinessInfo;
import com.dragon.tj.portal.entity.InstallInfo;
import com.dragon.tj.portal.entity.PageParam;
import com.dragon.tj.portal.service.BusinessService;
import com.dragon.tj.portal.service.InstallInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.List;

@RestController
@RequestMapping("/business")
@EnableScheduling
public class BusinessController {


    @Autowired
    private BusinessService businessService;

    @Autowired
    private InstallInfoService installInfoService;

    /*
     *    1. 我的业务查询 批量排序收藏
     * */
    @PostMapping("list")
    public R appList(@RequestBody PageParam<BusinessInfo> businessPage) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        return R.ok(businessService.installApp(businessPage,loginUser));
    }

    /*
     *  1-1安装业务
     * */
    @GetMapping("install/{appId}/{businessId}")
    public R install(@PathVariable("appId") Integer appId,
                     @PathVariable("businessId") Integer businessId) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        int install = businessService.install(appId, businessId, loginUser.getIdCard());

        if (install == 0){

            return R.failed("安装失败!!!");
        }
        return R.ok();
    }

    /*
     *  1-2卸载业务
     * */
    @GetMapping("uninstall/{appId}/{businessId}")
    public R uninstall(@PathVariable("appId") Integer appId,
                       @PathVariable("businessId") Integer businessId) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        return R.ok(businessService.uninstall(appId,businessId,loginUser.getIdCard()));
    }

    /***************************************/

    /*
     *    2. 类型查询
     * */
//    @GetMapping("types")
//    public R getAppType() {
//        return businessService.getType();
//    }

    /*
     *    3. 应用 CRUD
     * */
//    @SysLog("添加应用")
    @PostMapping("add")
    public R addApp(@RequestBody BusinessInfo businessInfo) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        int b = businessService.add(businessInfo, loginUser);
        if (b == 0){
            return R.failed("添加失败!!!");
        }

        return R.ok(b);
    }

    //    @SysLog("应用删除")
    @GetMapping("delete/{id}")
    public R delete(@PathVariable("id") Long id) {
        return R.ok(businessService.delete(id));
    }

    //    @SysLog("应用更新")
    @PostMapping("update")
    public R update(@RequestBody BusinessInfo businessInfo) {
        businessInfo.setUpdateTime(LocalDateTime.now());
        return R.ok(businessService.update(businessInfo));
    }

    /*
     *   批量更新收藏
     * */
    @PostMapping("batchUpdate")
    public R batchUpdate(@RequestBody List<InstallInfo> list) {
        installInfoService.batchUpdate(list,1);
        return R.ok();
    }


    //    @SysLog("通过ID获取应用")
    @GetMapping("info/{id}")
    public R detail(@PathVariable("id") Long id) {
        return R.ok(businessService.detail(id));
    }

    /*
     *   通过应用名称,应用事权单位名称搜索app
     * */
    @PostMapping("search")
    public R search(@RequestBody PageParam<BusinessInfo> businessInfoPage) {

        return R.ok(businessService.search(businessInfoPage));
    }


    /*
     *    4. 热门应用统计 定时、实时？
     * */
    @Scheduled(fixedDelay = 1000 * 60 * 60)
    public void appStatHandler(){

        businessService.updateAppStat();
    }

}

