package com.dragon.tj.portal.controller;


import com.alibaba.fastjson.JSON;
import com.dragon.tj.portal.auth.model.LoginUser;
import com.dragon.tj.portal.auth.util.SecurityUtils;
import com.dragon.tj.portal.common.base.R;
import com.dragon.tj.portal.common.dto.message.MessageInfoReq;
import com.dragon.tj.portal.common.dto.notice.NoticeInfoReq;
import com.dragon.tj.portal.component.message.MessageProducer;
import com.dragon.tj.portal.component.page.KafkaUtil;
import com.dragon.tj.portal.service.MessageInfoService;
import com.dragon.tj.portal.service.NoticeInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.concurrent.ExecutionException;

/**
 * <p>
 * 消息对接
 * </p>
 *
 * @author huey
 * @since 2023-06-15
 */
@Slf4j
@RestController
@RequestMapping("/api/message")
public class MessageCenterController {

    @Autowired
    private MessageInfoService messageInfoService;

    @Autowired
    private NoticeInfoService noticeInfoService;

    /**
     * @author huey China.
     * @Description : 信息发送
     * @Date Created in 2023/7/11 15:14
     */
    @PostMapping("/publish")
    public R publish(@Validated @RequestBody MessageInfoReq messageInfoReq) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        log.info("api/message/publish-req-is {}-{}", JSON.toJSONString(loginUser), JSON.toJSONString(messageInfoReq));
        return messageInfoService.push(messageInfoReq, loginUser);
    }

    /**
     * @author huey China.
     * @Description : 消息提醒发送
     * @Date Created in 2023/7/11 16:12
     */
    @PostMapping("/send")
    public R send(@Validated @RequestBody NoticeInfoReq noticeInfoReq) {
        log.info("api/message/send-req-is {}", JSON.toJSONString(noticeInfoReq));
        return noticeInfoService.send(noticeInfoReq);
    }

    @Autowired
    private KafkaUtil kafkaUtil;

    @GetMapping("/createTopic")
    public R createTopic(@RequestParam String appId,@RequestParam String topic) throws ExecutionException, InterruptedException {
        kafkaUtil.create(topic);

        return R.ok();
    }

    /**
    * @author huey China.
    * @Description : 获取信息和消息的待办和已办的个数
    * @Date Created in 2023/7/21 15:29
    */
    @GetMapping("/getCount")
    public R getCount() {
        return R.ok(messageInfoService.getCount());
    }


}

